/// <reference types="react" />
export declare enum AnchorType {
    Left = "anchor-left",
    Right = "anchor-right",
    Top = "anchor-top",
    Bottom = "anchor-bottom"
}
export declare enum CenterType {
    None = 0,
    Vertical = 1,
    HorizontalVertical = 2
}
export interface IPublicProps {
    id?: string;
    className?: string;
    style?: React.CSSProperties;
    scrollable?: boolean;
    trackSize?: boolean;
    centerContent?: CenterType;
    as?: string;
    children?: React.ReactNode;
    debug?: boolean;
}
export interface IPrivateProps {
    anchorSize?: string | number;
    anchor?: AnchorType;
    resizable?: boolean;
    order?: number;
}
export interface IAnchoredProps {
    size: number | string;
    order?: number;
}
export interface IResizableProps {
    handleSize?: number;
    overlayHandle?: boolean;
    minimumSize?: number;
    maximumSize?: number;
}
export declare type AllProps = IPublicProps & IPrivateProps & IResizableProps;
export interface IState {
    id: string;
    currentWidth: number;
    currentHeight: number;
    adjustedSize: number;
    spaceTakers: ISpaceTaker[];
    parsedSize?: number;
    left?: number;
    top?: number;
    right?: number;
    bottom?: number;
    width?: number | string;
    height?: number | string;
    debug: boolean;
}
export interface ISpaceContext {
    debug: boolean;
    level: number;
    width: number;
    height: number;
    spaceTakers: ISpaceTaker[];
    registerSpaceTaker: (spaceTaker: ISpaceTaker) => void;
    removeSpaceTaker: (id: string) => void;
    updateSpaceTakerAdjustedSize: (id: string, adjustedSize: number) => void;
    updateDebug: (id: string, debug: boolean) => void;
}
export interface ISpaceTaker {
    id: string;
    order: number;
    anchorType: AnchorType;
    size: number | string;
    adjustedSize: number;
}
export interface ISpaceInfo {
    width: number;
    height: number;
}
