import { createElement, Fragment, createRef, Component, createContext as createContext$1, useContext, useState, useRef, useEffect } from 'react';
import { ResizeSensor } from 'css-element-queries';
import { Guid } from 'guid-typescript';

var Centered = function (props) { return (createElement("div", { className: "spaces-centered" }, props.children)); };
var CenteredVertically = function (props) { return (createElement("div", { className: "spaces-centered-vertically" }, props.children)); };

var ClearFix = function (props) { return (createElement("div", { className: "spaces-clearfix" }, props.children)); };

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var AnchorType;
(function (AnchorType) {
    AnchorType["Left"] = "anchor-left";
    AnchorType["Right"] = "anchor-right";
    AnchorType["Top"] = "anchor-top";
    AnchorType["Bottom"] = "anchor-bottom";
})(AnchorType || (AnchorType = {}));
var CenterType;
(function (CenterType) {
    CenterType[CenterType["None"] = 0] = "None";
    CenterType[CenterType["Vertical"] = 1] = "Vertical";
    CenterType[CenterType["HorizontalVertical"] = 2] = "HorizontalVertical";
})(CenterType || (CenterType = {}));

var ResizeType;
(function (ResizeType) {
    ResizeType["Left"] = " resize-left";
    ResizeType["Right"] = " resize-right";
    ResizeType["Top"] = " resize-top";
    ResizeType["Bottom"] = " resize-bottom";
})(ResizeType || (ResizeType = {}));
var Resizable = /** @class */ (function (_super) {
    __extends(Resizable, _super);
    function Resizable(props) {
        var _this = _super.call(this, props) || this;
        _this.divElementRef = createRef();
        _this.originalMouseX = 0;
        _this.originalMouseY = 0;
        _this.startResize = function (e) {
            e.preventDefault();
            if (_this.props.type === ResizeType.Right) {
                _this.originalMouseX = e.pageX - _this.state.adjustedSize;
            }
            else if (_this.props.type === ResizeType.Left) {
                _this.originalMouseX = e.pageX + _this.state.adjustedSize;
            }
            else if (_this.props.type === ResizeType.Top) {
                _this.originalMouseY = e.pageY + _this.state.adjustedSize;
            }
            else if (_this.props.type === ResizeType.Bottom) {
                _this.originalMouseY = e.pageY - _this.state.adjustedSize;
            }
            window.addEventListener('mousemove', _this.resize);
            window.addEventListener('mouseup', _this.stopResize);
        };
        _this.resize = function (e) {
            var adjustment = 0;
            if (_this.props.type === ResizeType.Right) {
                adjustment = e.pageX - _this.originalMouseX;
            }
            else if (_this.props.type === ResizeType.Left) {
                adjustment = _this.originalMouseX - e.pageX;
            }
            else if (_this.props.type === ResizeType.Top) {
                adjustment = _this.originalMouseY - e.pageY;
            }
            else if (_this.props.type === ResizeType.Bottom) {
                adjustment = e.pageY - _this.originalMouseY;
            }
            if (adjustment < _this.props.minimumAdjust) {
                adjustment = _this.props.minimumAdjust;
            }
            if (_this.props.maximumAdjust && adjustment > _this.props.maximumAdjust) {
                adjustment = _this.props.maximumAdjust;
            }
            if (adjustment !== _this.state.adjustedSize) {
                _this.setState({ adjustedSize: adjustment });
                _this.props.onResize(adjustment);
            }
        };
        _this.stopResize = function () {
            window.removeEventListener('mousemove', _this.resize);
        };
        _this.state = {
            adjustedSize: 0
        };
        return _this;
    }
    Resizable.prototype.render = function () {
        return (createElement(Fragment, null,
            createElement("div", { ref: this.divElementRef, style: {
                    width: this.props.width,
                    height: this.props.height,
                    position: 'absolute',
                    zIndex: 9999
                }, className: "spaces-resize-handle" + this.props.type, onMouseDown: this.startResize })));
    };
    return Resizable;
}(Component));

var getSizeString = function (size) { return typeof (size) === "string" ? size : size + "px"; };
var isFilledSpace = function (props) { return !props.anchor; };
var isHorizontalSpace = function (props) {
    return props.anchor && (props.anchor === AnchorType.Left || props.anchor === AnchorType.Right);
};
var isVerticalSpace = function (props) {
    return props.anchor && (props.anchor === AnchorType.Top || props.anchor === AnchorType.Bottom);
};
var initialState = function (props) { return ({
    id: Guid.create().toString(),
    currentWidth: 0,
    currentHeight: 0,
    adjustedSize: 0,
    spaceTakers: [],
    parsedSize: typeof props.anchorSize === "string" ? 0 : props.anchorSize,
    left: props.anchor !== AnchorType.Right ? 0 : undefined,
    top: props.anchor !== AnchorType.Bottom ? 0 : undefined,
    right: props.anchor !== AnchorType.Left ? 0 : undefined,
    bottom: props.anchor !== AnchorType.Top ? 0 : undefined,
    width: isHorizontalSpace(props) ? props.anchorSize || 0 : undefined,
    height: isVerticalSpace(props) ? props.anchorSize || 0 : undefined,
    debug: props.debug !== undefined ? props.debug : false,
}); };
var createContext = function (state, setState, parent) {
    return {
        debug: parent ? (parent.debug ? true : state.debug) : state.debug,
        level: parent ? parent.level + 1 : 0,
        width: state.currentWidth,
        height: state.currentHeight,
        spaceTakers: state.spaceTakers,
        registerSpaceTaker: function (spaceTaker) {
            var existing = state.spaceTakers.find(function (t) { return t.id === spaceTaker.id; });
            if (!existing) {
                setState({
                    spaceTakers: state.spaceTakers.concat([spaceTaker])
                });
            }
            else {
                existing.order = spaceTaker.order;
                existing.anchorType = spaceTaker.anchorType;
                existing.size = spaceTaker.size;
            }
        },
        removeSpaceTaker: function (id) {
            setState({
                spaceTakers: state.spaceTakers.filter(function (t) { return t.id !== id; })
            });
        },
        updateSpaceTakerAdjustedSize: function (id, adjustedSize) {
            setState({
                spaceTakers: state.spaceTakers.map(function (t) {
                    return t.id === id ? __assign({}, t, { adjustedSize: adjustedSize }) :
                        t;
                })
            });
        },
        updateDebug: function (id, debug) {
            setState({
                spaceTakers: state.spaceTakers.map(function (t) {
                    return t.id === id ? __assign({}, t, { debug: debug }) :
                        t;
                })
            });
        }
    };
};
var applyResize = function (props, state, setState, parentContext, handleSize) {
    var resizeType = ResizeType.Left;
    var resizeHandleWidth;
    var resizeHandleHeight;
    if (parentContext && props.anchor && props.resizable) {
        switch (props.anchor) {
            case AnchorType.Left:
                resizeType = ResizeType.Right;
                resizeHandleWidth = handleSize;
                break;
            case AnchorType.Right:
                resizeType = ResizeType.Left;
                resizeHandleWidth = props.handleSize || 5;
                break;
            case AnchorType.Top:
                resizeType = ResizeType.Bottom;
                resizeHandleHeight = props.handleSize || 5;
                break;
            case AnchorType.Bottom:
                resizeType = ResizeType.Top;
                resizeHandleHeight = props.handleSize || 5;
                break;
        }
        return {
            resizeHandle: createElement(Resizable, { type: resizeType, width: resizeHandleWidth, height: resizeHandleHeight, minimumAdjust: -(state.parsedSize || 0) + (props.minimumSize || 20), maximumAdjust: props.maximumSize ? (props.maximumSize - (state.parsedSize || 0)) : undefined, onResize: function (adjustedSize) {
                    setState({ adjustedSize: adjustedSize });
                    parentContext.updateSpaceTakerAdjustedSize(state.id, adjustedSize);
                } }),
            resizeType: resizeType
        };
    }
    return { resizeHandle: null, resizeType: null };
};
var calculateSpace = function (props, state, setState, registerOnRemove, parentContext) {
    var outerStyle = {
        left: (state.left !== undefined ? "calc(" + state.left + "px)" : undefined),
        top: (state.top !== undefined ? "calc(" + state.top + "px)" : undefined),
        right: (state.right !== undefined ? "calc(" + state.right + "px)" : undefined),
        bottom: (state.bottom !== undefined ? "calc(" + state.bottom + "px)" : undefined),
        width: isHorizontalSpace(props) ?
            "calc(" + getSizeString(props.anchorSize || 0) + " + " + state.adjustedSize + "px)"
            : undefined,
        height: isVerticalSpace(props) ?
            "calc(" + getSizeString(props.anchorSize || 0) + " + " + state.adjustedSize + "px)"
            : undefined,
    };
    if (parentContext) {
        registerOnRemove(function () {
            parentContext.removeSpaceTaker(state.id);
        });
        var adjustedTop_1 = [];
        var adjustedLeft_1 = [];
        var adjustedRight_1 = [];
        var adjustedBottom_1 = [];
        [AnchorType.Left,
            AnchorType.Right,
            AnchorType.Bottom,
            AnchorType.Top]
            .forEach(function (a) {
            var spaceTakers = parentContext.spaceTakers
                .filter(function (t) { return t.anchorType === a; })
                .sort(function (a, b) { return a.order - b.order; });
            for (var i = 0; i < spaceTakers.length; i++) {
                var t = spaceTakers[i];
                if (t.id !== state.id) {
                    var adjustedSize = t.adjustedSize !== 0 ? " + " + t.adjustedSize + "px" : "";
                    if (isFilledSpace(props)) {
                        if (t.anchorType === AnchorType.Top) {
                            adjustedTop_1.push("" + getSizeString(t.size) + adjustedSize);
                        }
                        else if (t.anchorType === AnchorType.Left) {
                            adjustedLeft_1.push("" + getSizeString(t.size) + adjustedSize);
                        }
                        else if (t.anchorType === AnchorType.Bottom) {
                            adjustedBottom_1.push("" + getSizeString(t.size) + adjustedSize);
                        }
                        else if (t.anchorType === AnchorType.Right) {
                            adjustedRight_1.push("" + getSizeString(t.size) + adjustedSize);
                        }
                    }
                    else {
                        if (t.anchorType === AnchorType.Top && outerStyle.top !== undefined) {
                            adjustedTop_1.push("" + getSizeString(t.size) + adjustedSize);
                        }
                        else if (t.anchorType === AnchorType.Left && outerStyle.left !== undefined) {
                            adjustedLeft_1.push("" + getSizeString(t.size) + adjustedSize);
                        }
                        else if (t.anchorType === AnchorType.Bottom && outerStyle.bottom !== undefined) {
                            adjustedBottom_1.push("" + getSizeString(t.size) + adjustedSize);
                        }
                        else if (t.anchorType === AnchorType.Right && outerStyle.right !== undefined) {
                            adjustedRight_1.push("" + getSizeString(t.size) + adjustedSize);
                        }
                    }
                }
                else {
                    break;
                }
            }
        });
        [
            { adjusted: adjustedTop_1, setter: function (value) { return outerStyle.top = value; } },
            { adjusted: adjustedBottom_1, setter: function (value) { return outerStyle.bottom = value; } },
            { adjusted: adjustedLeft_1, setter: function (value) { return outerStyle.left = value; } },
            { adjusted: adjustedRight_1, setter: function (value) { return outerStyle.right = value; } }
        ].map(function (x) { return x.adjusted.length > 0 ? x.setter("calc(" + x.adjusted.join(" + ") + ")") : null; });
        if (props.anchor) {
            parentContext.registerSpaceTaker({
                id: state.id,
                order: props.order || 1,
                anchorType: props.anchor,
                size: props.anchorSize || 0,
                adjustedSize: 0
            });
        }
    }
    var currentContext = createContext(state, setState, parentContext);
    var handleSize = props.handleSize || 5;
    var overlayHandle = props.overlayHandle !== undefined ? props.overlayHandle : true;
    var children = props.children;
    var resize = applyResize(props, state, setState, parentContext, handleSize);
    if (props.centerContent === CenterType.Vertical) {
        children = createElement(CenteredVertically, null, children);
    }
    else if (props.centerContent === CenterType.HorizontalVertical) {
        children = createElement(Centered, null, children);
    }
    var innerStyle = __assign({}, props.style, {
        left: resize.resizeType === ResizeType.Left && !overlayHandle ? handleSize : undefined,
        top: resize.resizeType === ResizeType.Top && !overlayHandle ? handleSize : undefined,
        right: resize.resizeType === ResizeType.Right && !overlayHandle ? handleSize : undefined,
        bottom: resize.resizeType === ResizeType.Bottom && !overlayHandle ? handleSize : undefined
    });
    return {
        currentContext: currentContext,
        resizeHandle: resize.resizeHandle,
        resizeType: resize.resizeType,
        outerStyle: outerStyle,
        innerStyle: innerStyle,
        children: children
    };
};

var SpaceContext = createContext$1(null);
var SpaceInfoContext = createContext$1(null);

var Fill = function (props) { return createElement(Space, __assign({}, props)); };
var Top = function (props) { return createElement(Space, __assign({}, props, { anchor: AnchorType.Top, anchorSize: props.size })); };
var Left = function (props) { return createElement(Space, __assign({}, props, { anchor: AnchorType.Left, anchorSize: props.size })); };
var Bottom = function (props) { return createElement(Space, __assign({}, props, { anchor: AnchorType.Bottom, anchorSize: props.size })); };
var Right = function (props) { return createElement(Space, __assign({}, props, { anchor: AnchorType.Right, anchorSize: props.size })); };
var TopResizable = function (props) { return createElement(Space, __assign({}, props, { anchor: AnchorType.Top, anchorSize: props.size, resizable: true })); };
var LeftResizable = function (props) { return createElement(Space, __assign({}, props, { anchor: AnchorType.Left, anchorSize: props.size, resizable: true })); };
var BottomResizable = function (props) { return createElement(Space, __assign({}, props, { anchor: AnchorType.Bottom, anchorSize: props.size, resizable: true })); };
var RightResizable = function (props) { return createElement(Space, __assign({}, props, { anchor: AnchorType.Right, anchorSize: props.size, resizable: true })); };
var Space = function (props) {
    var parentContext = useContext(SpaceContext);
    var _a = useState(initialState(props)), state = _a[0], changeState = _a[1];
    var divElementRef = useRef();
    var resizeSensor = useRef(undefined);
    var onRemove = useRef(undefined);
    var setState = function (stateDelta) { return changeState(function (prev) { return (__assign({}, prev, stateDelta)); }); };
    // Deal with property changes to size / anchoring 
    useEffect(function () {
        setState({
            parsedSize: typeof props.anchorSize === "string" ? 0 : props.anchorSize,
            left: props.anchor !== AnchorType.Right ? 0 : undefined,
            top: props.anchor !== AnchorType.Bottom ? 0 : undefined,
            right: props.anchor !== AnchorType.Left ? 0 : undefined,
            bottom: props.anchor !== AnchorType.Top ? 0 : undefined,
            width: isHorizontalSpace(props) ? props.anchorSize || 0 : undefined,
            height: isVerticalSpace(props) ? props.anchorSize || 0 : undefined,
            debug: props.debug !== undefined ? props.debug : false,
        });
    }, [props.anchor, props.anchorSize, props.debug]);
    // Setup / cleanup
    useEffect(function () {
        if (divElementRef.current) {
            if (props.trackSize) {
                resizeSensor.current = new ResizeSensor(divElementRef.current, function (size) { return setState({ currentWidth: size.width, currentHeight: size.height }); });
            }
            var currentRect = divElementRef.current.getBoundingClientRect();
            setState({
                parsedSize: !state.parsedSize ?
                    (isHorizontalSpace(props) ? currentRect.width : currentRect.height) :
                    state.parsedSize,
                currentWidth: currentRect.width,
                currentHeight: currentRect.height
            });
        }
        var cleanup = function () {
            if (resizeSensor.current) {
                resizeSensor.current.detach();
                resizeSensor.current = undefined;
            }
            if (onRemove.current) {
                onRemove.current();
                onRemove.current = undefined;
            }
        };
        return cleanup;
    }, []);
    var debug = parentContext ? parentContext.debug : 
        props.debug !== undefined ? props.debug : false;
    var _b = calculateSpace(props, state, setState, function (handler) { onRemove.current = handler; }, parentContext), currentContext = _b.currentContext, outerStyle = _b.outerStyle, innerStyle = _b.innerStyle, resizeType = _b.resizeType, resizeHandle = _b.resizeHandle, children = _b.children;
    var outerClasses = [
        "spaces-space",
        props.anchor || '',
        resizeType || '',
        props.scrollable ? "scrollable" : '',
        props.className ? props.className + "-container" : '',
        debug ? 'debug' : ''
    ];
    var innerClasses = [
        "spaces-space-inner",
        props.className ? props.className : ''
    ];
    return (createElement(Fragment, null,
        createElement(SpaceContext.Provider, { value: currentContext },
            createElement(SpaceInfoContext.Provider, { value: { width: Math.floor(state.currentWidth), height: Math.floor(state.currentHeight) } }, createElement(props.as || 'div', {
                id: props.id,
                ref: divElementRef,
                className: outerClasses.join(' '),
                style: outerStyle
            }, createElement(Fragment, null,
                resizeHandle,
                createElement("div", { className: innerClasses.join(' '), style: innerStyle }, children)))))));
};

var Fixed = function (props) {
    var style = __assign({}, props.style, {
        width: props.width,
        height: props.height
    });
    return (createElement("div", { className: "spaces-fixedsize-layout" + (props.className ? " " + props.className : ""), style: style },
        createElement(Fill, null, props.children)));
};

var ViewPort = function (props) { return (createElement("div", { className: "spaces-fullpage-layout" + (props.className ? " " + props.className : ""), style: {
        left: props.left || 0,
        top: props.top || 0,
        right: props.right || 0,
        bottom: props.bottom || 0
    } },
    createElement(Fill, null, props.children))); };

var Info = function (props) { return (createElement(SpaceInfoContext.Consumer, null, function (info) { return props.children(info); })); };

export { AnchorType, Bottom, BottomResizable, CenterType, Centered, CenteredVertically, ClearFix, Fill, Fixed, Info, Left, LeftResizable, Right, RightResizable, Top, TopResizable, ViewPort };
//# sourceMappingURL=server.js.map
